@@ -1,3 +1,9 @@
+#
+# $Id: dTemplate.pm 132 2006-10-21 15:52:44Z dlux $
+# 
+# $URL: http://svn.dlux.hu/public/dTemplate/trunk/dTemplate.pm $ 
+#
+
 package dTemplate;
 use strict;
 use DynaLoader;
@@ -7,7 +13,7 @@ use vars qw($VERSION @ISA %ENCODERS $ENCODERS %parse
 
 @ISA = qw(DynaLoader);
 
-$VERSION             = '2.2.2';
+$VERSION             = '2.4.2';
 $START_DELIMITER     = '\$';
 $END_DELIMITER       = '\$';
 $VAR_PATH_SEP        = '\.';
@@ -20,17 +26,25 @@ dTemplate->bootstrap($VERSION);
 
 # Constructors ...
 
-sub define { my $obj=shift; ((ref($obj) || $obj)."::Template")->new(@_); };
-sub choose { my $obj=shift; ((ref($obj) || $obj)."::Choose")->new(@_); };
-*select=*choose;
-sub text   { my $obj=shift; ((ref($obj) || $obj)."::Template")->new_raw(@_); };
+sub new {
+    my $obj = shift;
+    my $type = shift or die "Invalid constructor call. Use: new dTemplate type => ...";
+    return ((ref $obj || $obj)."::Template")->new(@_) if        $type eq "file";
+    return ((ref $obj || $obj)."::Choose")->new(@_) if          $type eq "choose" || $type eq "select";
+    return ((ref $obj || $obj)."::Template")->new_raw(@_) if    $type eq "text";
+    die "Invalid type in dTemplate constructor: $type";
+}
+
+sub define { shift->new(file   => @_) }
+sub choose { shift->new(choose => @_) }
+sub select { shift->new(choose => @_) }
+sub text   { shift->new(text   => @_) }
+
 sub encode { 
-  my $encoder=shift();
+  my $encoder = shift;
   return $ENCODERS{$encoder}->(shift());
 };
 
-$parse{''} = sub { shift };
-
 $ENCODERS{''}   = sub { shift() };
 
 $ENCODERS{u}  = sub { 
@@ -86,9 +100,13 @@ sub spf {
 
 *ENCODERS = *dTemplate::ENCODERS;
 
-sub filename { 0 };
-sub text     { 1 };
-sub compiled { 2 };
+sub FILENAME  { 0 };
+sub TEXT      { 1 };
+sub COMPILED  { 2 };
+sub PARSEHASH { 3 };
+
+# use this constant to determinene the last field for subclassing dTemplate
+sub LAST_FIELD { PARSEHASH };
 
 sub new { my ($class,$filename)=@_;
     return undef if ! -r $filename;
@@ -98,21 +116,21 @@ sub new { my ($class,$filename)=@_;
 
 sub new_raw { my $class=shift;
   my $txt=shift;
-  my $s=[undef, ref($txt) ? $txt : \$txt];
+  my $s=[undef, (ref($txt) ? $txt : \$txt)];
   bless ($s,$class);
 };
 
 sub style  { return undef };
 
 sub compile { my $s=shift;
-    return if $s->[compiled];
+    return if $s->[COMPILED];
     $s->load_file;
 
     # template parsing 
 
     my %varhash;
     my @comp=({});
-    ${ $s->[text] } =~ s{ (.*?) ( 
+    ${ $s->[TEXT] } =~ s{ (.*?) ( 
         (?:$dTemplate::START_DELIMITER) ( (?:\w|(?:$dTemplate::VAR_PATH_SEP))* ) 
         ( (?:$dTemplate::PRINTF_SEP) (.*?[\w]) )? 
         ( (?:$dTemplate::ENCODER_SEP) (.*?) )?
@@ -127,8 +145,9 @@ sub compile { my $s=shift;
             $clast->{text} .= $pre;
             if ($varname) {
                 $clast->{full_matched} = $full_matched;
-                my ($varn, @varp) = split (/$dTemplate::VAR_PATH_SEP/, 
+                my (@varp) = split (/$dTemplate::VAR_PATH_SEP/, 
                     $varname);
+                my $varn = $varp[0];
                 $clast->{varn} = $varn;
                 $varhash{$varn}++;
                 $clast->{varp} = \@varp;
@@ -138,7 +157,7 @@ sub compile { my $s=shift;
             };
         };
         "";
-    }gxsce;
+    }gxse;
 
     # assigning ID-s for variables
 
@@ -190,24 +209,26 @@ sub compile { my $s=shift;
         }
     }
 
-    $s->[compiled] = $compiled;
-    $s->[text]=undef; # free up some memory
+    $s->[COMPILED] = $compiled;
+    $s->[TEXT]=undef; # free up some memory
 };
 
 sub load_file { my $s=shift;
-  return if $s->[compiled] || defined $s->[text] || !defined $s->[filename];
-  if (!open(FILE,$s->[filename])) {
-    warn "Cannot load template file: ".$s->[filename];
-    $s->[text]=\"";
+    return if $s->[COMPILED] || defined $s->[TEXT] || !defined $s->[FILENAME];
+    if (!open(FILE,$s->[FILENAME])) {
+        warn "Cannot load template file: ".$s->[FILENAME];
+        $s->[TEXT]=\"";
     close (FILE);
     return;
-  };
-  local $/=undef;
-  my $text=<FILE>;
-  $s->[text]=\$text;
-  close (FILE);
+    };
+    local $/=undef;
+    my $text=<FILE>;
+    $s->[TEXT]=\$text;
+    close (FILE);
 };
 
+sub parsehash: lvalue { shift->[PARSEHASH] ||= {} }
+
 package dTemplate::Choose;
 use strict;
 
